/* Developed for Lenovo Educational Board Pass 2
   © Lenovo Created by Joshua Grosser
   v. 1.0 */

#include <Adafruit_NeoPixel.h>

#define NEOPIXEL_PIN 45 // Neopixel is on pin 45
#define NEOPIXEL_COUNT 1 // There is only 1 RGB LED on DOS
#define NEOPIXEL_INDEX 0 // RGB LED is at index 0

// Create a NeoPixel object
Adafruit_NeoPixel Pixel(NEOPIXEL_COUNT, NEOPIXEL_PIN, NEO_GRB + NEO_KHZ800);
// set computer guess and number of trials
String finalWord = "";
int trials = 0;

// array of words that can be chosen
static String wordle_answers[] = {
  "honor",
  "rebut",
  "sassy",
  "awake",
  "blush",
  "focal",
  "today",
  "evade",
  "naval",
  "serve",
  "dwarf",
  "model",
  "karma",
  "stink",
  "grade",
  "quiet",
  "bench",
  "feign",
  "major",
  "sugar",
  "fresh",
  "crust",
  "marry",
  "react",
  "pride",
  "floss",
  "helix",
  "croak",
  "staff",
  "paper",
  "unfed",
  "whelp",
  "outdo",
  "adobe",
  "crazy",
  "repay",
  "digit",
  "crate",
  "spike",
  "mimic",
  "pound",
  "linen",
  "unmet",
  "flesh",
  "forth", 
  "first",
  "stand",
  "belly",
  "ivory",
  "print",
  "yearn",
  "drain",
  "bribe",
  "panel",
  "flume",
  "agree",
  "error",
  "swirl",
  "argue",
  "bleed",
  "delta",
  "flick",
  "front",
  "shrub",
  "biome",
  "start",
  "greet",
  "goner",
  "round",
  "audit",
  "gamma",
  "labor",
  "civic",
  "forge",
  "corny",
  "basic",
  "salad",
  "spicy",
  "spray",
  "essay",
  "spend",
  "motor",
  "alone",
  "hatch",
  "hyper",
  "ought",
  "belch",
  "pilot",
  "comet",
  "jaunt",
  "abyss",
  "growl",
  "dozen",
  "erode",
  "click",
  "world",
  "gouge",
  "briar",
  "chill",
  "clock",
};

// array that displays the correct guessed letters
String guessed[] = {
  "_",
  "_",
  "_",
  "_",
  "_",
};

void setup() {
  // sets baud rate
  Serial.begin(115200);
  // initialize Neopixel
  Pixel.begin();
  Pixel.clear();
  Pixel.show();
  // picks a random word
  int r = random(1,101);
  finalWord = wordle_answers[r];
  Serial.println();
  Serial.println();
  Serial.println("You will have 6 attempts to guess a 5 letter english word.");
  Serial.println();
  Serial.println("WORDLE");
}

void loop() {
  if(trials < 6) // only have 6 guesses
  {
    while(Serial.available() > 0)
    {
      // reads the wordle guess
      String curGuess = Serial.readString(); 
      Serial.println();
      // A newline character indicates end of input
      if(curGuess[5] == '\n')
      {
        Serial.print("You Guessed: "); Serial.println(curGuess);
        // displays the neopixel colors whether the letter is correct or not
        for(int i = 0; i < 5; i++)
        {
          // displays green if in word in correct spot
          if(curGuess[i] == finalWord[i])
          {
            guessed[i] = finalWord[i];
            Pixel.setPixelColor(NEOPIXEL_INDEX, 0, 255, 0);
            Pixel.show();
            delay(200);
            Pixel.setPixelColor(NEOPIXEL_INDEX, 0, 0, 0);
            Pixel.show();
            delay(500);
          }
          else
          {
            bool found = false;
            for(int j = 0; j < 5; j++)
            {
              // displays blue if in word and not in correct spot
              if(curGuess[i] == finalWord[j])
              {
                Serial.print(finalWord[j]);
                Serial.println(" is in the word");
                Pixel.setPixelColor(NEOPIXEL_INDEX, 0, 0, 255);
                Pixel.show();
                delay(200);
                Pixel.setPixelColor(NEOPIXEL_INDEX, 0, 0, 0);
                Pixel.show();
                delay(500);
                found = true;
              }
              // displays red if not in word
              else if (curGuess[i] != finalWord[j] && j == 4 && !found)
              {
                Pixel.setPixelColor(NEOPIXEL_INDEX, 255, 0, 0);
                Pixel.show();
                delay(200);
                Pixel.setPixelColor(NEOPIXEL_INDEX, 0, 0, 0);
                Pixel.show();
                delay(500);
              }
            }
          }
        }
        // changes to display correctly guessed letters  
        Serial.printf("%s %s %s %s %s", guessed[0], guessed[1], guessed[2], guessed[3], guessed[4]);
        Serial.println();
        trials++;
        if(curGuess[0] == finalWord[0] && curGuess[1] == finalWord[1] && curGuess[2] == finalWord[2] && curGuess[3] == finalWord[3] && curGuess[4] == finalWord[4])
        { // win condition
          Serial.println();
          Serial.println("Congratulations! You have guessed correctly!");
          trials = 7;
        }
      }
    } 
  }
  else if (trials == 6) 
  { // ran out of attempts
    Serial.println();
    Serial.println("Sorry you have run out of attempts!");
    Serial.print("The word was: "); Serial.println(finalWord);
    trials++;
  }
  else
  { // picks a new word for next game
    int r = random(1,101);
    finalWord = wordle_answers[r];
    trials = 0;
    restart();
  }
}

// resets the wordle game
void restart()
{
  for(int i = 0; i < 5; i++)
  {
    guessed[i] = "_";
  }
  Serial.println();
  Serial.println();
  Serial.println("You will have 6 attempts to guess a 5 letter english word.");
  Serial.println();
  Serial.println("WORDLE");
}
